/*
 * skillchains-avatars.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the FFXI Skillchains Tool.
 *
 * The FFXI Skillchains Tool is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The FFXI Skillchains Tool is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the FFXI Skillchains Tool; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Controls the UI for selecting a group of avatars.
 */

var rownum = window.arguments[0];

function init() {
    var container = document.getElementById("avatar-group");
    // Grab the DB off the opener
    if (window.opener.SkillChainDB) {
	var curAvatars = window.arguments[1];
	if (curAvatars == null)
	    curAvatars = [];
	for (var a in window.opener.SkillChainDB.avatars) {
	    var avatar = window.opener.SkillChainDB.avatars[a];
	    var checkbox = document.createElement("checkbox");
	    checkbox.setAttribute("label", avatar.name);
	    checkbox.setAttribute("id", "a-" + a);
	    checkbox.setAttribute("oncommand", "updateAll()");
	    for (var ca in curAvatars) {
		if (curAvatars[ca] == avatar) {
		    checkbox.setAttribute("checked", "true");
		    break;
		}
	    }
	    container.appendChild(checkbox);
	}
	var checkbox = document.createElement("checkbox");
	checkbox.setAttribute("label", window.opener.getLocalString("avatar.all"));
	checkbox.setAttribute("id", "allcheck");
	checkbox.setAttribute("oncommand", "checkAll(this.checked)");
	container.appendChild(checkbox);
	updateAll();
    } else {
	// dope
	var label = document.createElement("label");
	label.setAttribute("value", "Unable to find DB on opener.");
	container.appendChild(label);
    }
}

function updateAll() {
    var all = true;
    var none = true;
    for (var a in window.opener.SkillChainDB.avatars) {
	if (document.getElementById("a-" + a).checked) {
	    none = false;
	    if (!all)
		break;
	} else {
	    all = false;
	    if (!none)
		break;
	}
    }
    dump("All checkbox status [none=" + none + ";all=" + all + "]\n");
    // Despite tri-state support being listed for M10, it appears that five
    // years later it is not actually implemented.  Oh well - instead we
    // check the box only when all are selected and ignore the fact that
    // there's really an intermediate state.
    document.getElementById("allcheck").checked = all;
}
function checkAll(check) {
    for (var a in window.opener.SkillChainDB.avatars) {
	document.getElementById("a-" + a).checked = check;
    }
}

function acceptChanges() {
    if (window.opener.SkillChainDB) {
	// get the array of avatars
	var avatars = new Array();
	for (var a in window.opener.SkillChainDB.avatars) {
	    if (document.getElementById("a-" + a).checked) {
		avatars.push(window.opener.SkillChainDB.avatars[a]);
	    }
	}
	window.opener.avatarSelectCallback(rownum, avatars);
    } else {
	// just die
    }
}

function dropChanges() {
    if (window.opener.avatarSelectCallback)
	window.opener.avatarSelectCallback(rownum, null);
}
